-- Combo-box hack
local function list2(params)
	assert(type(params) == "table", "bad argument #1 to 'list2' (table expected, got "..type(params)..")")
	-- the Action callback is only applied to the textbox
	params.expand = params.expand or 'NO'
	params.list = params.list or {}
	params.font = params.font or Font.Default
	params.value = params.value or ''
	params.active = params.active or 'YES'

	local dropdown, textbox, zbox
	
	dropdown = iup.list{dropdown = 'YES', size = params.size, font = params.font, expand = params.expand, fgcolor = '0 0 0 0', visible_items = params.visible_items,
		action = function(self, t, i, v) textbox.value = t; iup.SetFocus(textbox); if (params.action) then textbox:action(nil, t) end end
	}
	for i, v in ipairs(params.list) do dropdown[i] = v; if (v == params.value) then dropdown.value = i end; end
	if (not params.size) then 
		iup.Map(dropdown)
		params.size = tonumber(dropdown.size:match("(%d+)x")) - params.font
	else
		params.size = params.size - params.font
	end
	textbox = iup.text{size = params.size, font = params.font, bgcolor = params.bgcolor, expand = params.expand, value = params.value,
		action = function(self, c, after) if (params.action) then params.action(self, c, after) end end
	}
	zbox = iup.zbox{
		--dropdown must be before textbox within the zbox to create the proper z-order
		dropdown,
		textbox,
		all = 'YES',
		alignment = 'NW',
	}

	function zbox:Activate(b)
		if (b) then
			dropdown.active = "YES"
			textbox.active = "YES"; textbox.bgcolor = params.bgcolor
		else
			dropdown.active = "NO"
			textbox.active = "NO"; textbox.bgcolor = params.bginactive
		end
	end

	function zbox:Clear()
		dropdown.value = 0
		textbox.value = ""
	end
	
	function zbox:SetAction(f)
		assert(type(f) == "function", "bad argument #1 to 'SetAction' (function expected, got "..type(f)..")")
		params.action = f
	end

	if (params.active:upper() == "NO") then zbox:Activate(false) end
	
	return zbox, textbox, dropdown
end

-- Image select dropdown list
local function list3(params)
	params = params or {}
	params.control = 'YES'
	--params.expand = 'YES'
	
	local controls, items = {}, {}
	local colors, num_colors = {}, 0
	local actioncb, createitemfunc, sortfunc

	local list = iup.stationsublist(params)

	local function create_subdlg(ctrl)
		assert(type(ctrl) == 'userdata' and iup.GetType(ctrl):match('box'))
		ctrl = ctrl or iup.hbox{}
		local dlg = iup.dialog{
			ctrl,
			border = 'NO', menubox = 'NO', resize = 'NO', active = 'NO'
		}
		return dlg
	end
	
	-- Custom methods
	function list:AddItems(itemstbl)
		assert(type(itemstbl) == 'table')
		items = itemstbl
	end

	function list:ClearItems()
		items = {}
	end
	
	function list:ClearList()
		self[1] = nil -- why?
		for i, subdlg in ipairs(controls) do
			iup.Detach(subdlg)
			iup.Destroy(subdlg)
			controls[i] = nil
		end
	end
	
	function list:PopulateList()
		self:ClearList()
		
		--if (sortfunc) then table.sort(controls, sortfunc) end

		for i, item in ipairs(items) do
			if (createitemfunc) then item = createitemfunc(item) end

			local subdlg = create_subdlg(item)
			subdlg:map()
			table.insert(controls, subdlg)
		end

		for i, subdlg in ipairs(controls) do
			if (num_colors > 0) then subdlg.bgcolor = colors[(i % num_colors) + 1] end
			iup.Append(self, subdlg)
		end

		self:map()

		self[1] = 1 -- why?
	end
	
	function list:SetActionCB(func)
		-- Argument should be a function that will be executed after appropriate color changes have been applied
		-- this function should receive one argument, the selected index
		if (func) then assert(type(func) == 'function') end
		actioncb = func
	end
	
	function list:SetCreateItemFunc(func)
		-- Function should receive an argument identifying the item and return a single box that will be put into a sub-dialog template
		if (func) then assert(type(func) == 'function') end
		createitemfunc = func
	end
	
	function list:SetColors(colortbl)
		-- Function should receive a table with numeric indexes. [0] will be used to indicate the color of the selected item.
		assert(type(colortbl) == 'table')
		if (not colortbl[0] and colortbl[1]) then
			-- if no select color, invert the first color
			local mask = bitlib.bnot(255)
			local selcolorparts = {}
			for v in colortbl[1]:gmatch('%S+') do
				local num = tonumber(v)
				if (v) then num = (bitlib.bnot(v) - mask) else num = v end
				table.insert(selcolorparts, num)
			end
			colortbl[0] = table.concat(selcolorparts, ' ')
		end
		num_colors = #colortbl
		colors = colortbl
		printtable(colors)
		--test.print(num_colors)
	end

	function list:SetSelected(index)
		if (index) then 
			assert(type(index) == 'number')
			self:action(true, index, 1) -- lists of controls return nil in the 1st parameter, setting to non-nil/false for init
		end
		if (self.vo_listsel) then self:action(true, tonumber(self.vo_listsel), 0) end
		self.vo_listsel = index
	end
	
-- 	function list:SetSortFunc(func)
-- 		if (func) then assert(type(func) == 'function') end
-- 		sortfunc = func
-- 	end

	-- Callbacks
	function list:action(text, index, state)
		--test.varprint(' / ', 'text', text, 'index', index, 'state', state, 'vo_listsel', self.vo_listsel)
		if (state == 1) then
			controls[index].bgcolor = colors[0]
			if (index ~= tonumber(self.vo_listsel)) then
				self.cursel = index -- necessary because vo_listsel doesn't get updated until the callback finishes
				if (text ~= true and actioncb) then return actioncb(self, text, index, state) end
			else
				-- Not going to allow deselection.  An image must be chosen.
			end

		else
			controls[index].bgcolor = colors[(index % num_colors) + 1]

		end
	end
	
	function list:multiselect_cb(value)
		--test.varprint('', 'value', value)
	end
	
	function list:edit_cb(c, after)
		--test.varprint(' / ', 'char_code', c, 'after', after)
	end
	
	function list:caret_cb(row, col)
		--test.varprint(' / ', 'row', row, 'col', col)
	end
	
	return list
end

-- 3 buttons, save/discard/cancel type 
local function msgdlgtemplate3(msg, button1text, button1action, button2text, button2action, button3text, button3action, dlgbgcolor, alignment)
	local dlg
	local infolabel = iup.label{font=Font.H1,title=msg or "Title", alignment=alignment}
	local button1 = iup.stationbutton{title=button1text or "Save", action=button1action}
	local button2 = iup.stationbutton{title=button2text or "Discard", action=button2action}
	local button3 = iup.stationbutton{title=button3text or "Cancel", action=button3action}

	dlg = iup.dialog{
		iup.hbox{
			iup.fill{},
			iup.vbox{
				iup.fill{},
				iup.stationhighopacityframe{
					bgcolor=dlgbgcolor,
					iup.stationhighopacityframebg{
						bgcolor=dlgbgcolor,
						iup.vbox{
							infolabel,
							iup.hbox{
								iup.fill{},
								button1,
								iup.fill{},
								button2,
								iup.fill{},
								button3,
								iup.fill{},
							},
						},
						expand="NO",
					},
				},
				iup.fill{},
			},
			iup.fill{},
		},
		--defaultenter = button1,
		defaultesc = button3,
		fullscreen="YES",
		bgcolor = "0 0 0 128 *",
		topmost="YES",
	}

	function dlg:SetMessage(msg, button1text, callback1, button2text, callback2, button3text, callback3, alignment)
		infolabel.alignment = alignment or "ALEFT"
		infolabel.title = msg or ""
		button1.title = button1text or "Save"
		if callback1 then button1.action = callback1 end
		button2.title = button2text or "Discard"
		if callback2 then button2.action = callback2 end
		button3.title = button3text or "Cancel"
		if callback3 then button3.action = callback3 end
		dlg.size = nil
	end

	return dlg
end

return list2, list3, msgdlgtemplate3